const fs = require('fs')
const csv = require('csv-parser')

function readCSV (filePath) {
  return new Promise((resolve, reject) => {
    const results = []
    fs.createReadStream(filePath)
      .pipe(csv())
      .on('data', data => {
        const timestamp = +data['start timestamp [ns]'].slice(0, 13) + 120
        const duration = +data['duration [ms]']
        const left = (+data['fixation x [px]'] - 770) * 1.325
        const top = (+data['fixation y [px]'] - 610) * 1.325
        const index = data['fixation id']
        results.push({ timestamp, duration, left, top, index })
      })
      .on('end', () => resolve(results))
      .on('error', error => reject(error))
  })
}

const filePath = 'fixations.csv'

readCSV(filePath)
  .then(results => {
    console.log(JSON.stringify(results))
  })
  .catch(err => {
    console.error(err)
  })
