import '@babylonjs/core/Debug/debugLayer'
import '@babylonjs/inspector'
import { Engine } from '@babylonjs/core'
import Papa from 'papaparse'
import { createScene } from './renderScene.js'

//Grab DOM element where we will attach our canvas. #app is the id assigned to an empty <div> in our index.html
const app = document.querySelector('#app')

//Create a canvas element and append it to #app div
const form = document.createElement('form')
form.id = 'uploadForm'

const fixationsInput = document.createElement('input')
fixationsInput.type = 'file'
fixationsInput.id = 'fixations'

const fileIndexLabel = document.createElement('label')
fileIndexLabel.textContent = 'Timeset Offset: '
const fileIndexInput = document.createElement('input')
fileIndexInput.type = 'number'
fileIndexInput.id = 'fileIndex'
fileIndexInput.value = '0'
fileIndexInput.style.marginRight = '15px'

const replayButton = document.createElement('button')
replayButton.type = 'button'
replayButton.textContent = 'Replay'
replayButton.addEventListener('click', replay)

form.appendChild(fixationsInput)
form.appendChild(fileIndexLabel)
form.appendChild(fileIndexInput)
form.appendChild(replayButton)

app.appendChild(form)

//initialize babylon engine, passing in our target canvas element, and create a new scene
const canvas = document.createElement('canvas')
app.appendChild(canvas)
const babylonEngine = new Engine(canvas, true)

//create a scene object using our engine
let scene

//Listen for window size changes and resize the scene accordingly
window.addEventListener('resize', function () {
  babylonEngine.resize()
})

// hide/show the Inspector
window.addEventListener('keydown', ev => {
  // Shift+Ctrl+Alt+I
  if (ev.shiftKey && ev.ctrlKey && ev.altKey && ev.keyCode === 73) {
    if (scene.debugLayer.isVisible()) {
      scene.debugLayer.hide()
    } else {
      scene.debugLayer.show()
    }
  }
})

function replay () {
  const fixationsFile = fixationsInput.files[0]

  // 获取三个输入框的值
  const fileIndex = parseFloat(document.getElementById('fileIndex').value) || 0

  const readFixationsFile = new Promise((resolve, reject) => {
    const fixationsReader = new FileReader()
    fixationsReader.onload = function (event) {
      try {
        // 将三个偏移值传递给 readCSVFromString 函数
        const fixations = readCSVFromString(event.target.result)
        resolve(fixations)
      } catch (error) {
        reject(error)
      }
    }
    fixationsReader.onerror = reject
    fixationsReader.readAsText(fixationsFile)
  })

  readFixationsFile
    .then(fixations => {
      console.log(fixations)

      return createScene(
        babylonEngine,
        fixations,
        fileIndex
      )
    })
    .then(s => {
      scene = s

      // 设置渲染循环
      babylonEngine.runRenderLoop(() => {
        scene.render()
      })
    })
    .catch(error => {
      console.error('文件读取或场景创建出错:', error)
    })
}

function readCSVFromString (csvString) {
  const results = Papa.parse(csvString, {
    header: true,
    skipEmptyLines: true
  })

  const processedResults = results.data.map(data => {
    const timestamp = +data['start timestamp [ns]'].slice(0, 13)
    const duration = +data['duration [ms]']
    const left = (+data['fixation x [px]'] - 610) * 1.325
    const top = (+data['fixation y [px]'] - 770) * 1.325
    const index = data['fixation id']
    return { timestamp, duration, left, top, index }
  })

  return processedResults
}
