import '@babylonjs/core/Debug/debugLayer'
import '@babylonjs/inspector'
import { Engine, Quaternion, Vector3, WebXRFeatureName } from '@babylonjs/core'
import { createScene } from './scene.js'
import calibrate from './calibrate'
import createQuestionnaire from './questionnaire'
import { createEnviroment } from './utils'

//Grab DOM element where we will attach our canvas. #app is the id assigned to an empty <div> in our index.html
const app = document.querySelector('#app')
//Create a canvas element and append it to #app div
const canvas = document.createElement('canvas')
app.appendChild(canvas)

//initialize babylon engine, passing in our target canvas element, and create a new scene
const babylonEngine = new Engine(canvas, true, {
  preserveDrawingBuffer: true,
  stencil: true,
  disableWebGL2Support: false
})

//create a scene object using our engine
let scene
createScene(babylonEngine)
  .then(s => {
    scene = s

    return scene.createDefaultXRExperienceAsync({
      disableTeleportation: true
    })
  })
  .then(defaultXRExperience => {
    if (!defaultXRExperience.baseExperience) {
      console.log('No XR')
    } else {
      const featureManager = defaultXRExperience.baseExperience.featuresManager

      if (!featureManager) {
        console.log('No Feature Manager')
      } else {
        defaultXRExperience.baseExperience.featuresManager.enableFeature(
          WebXRFeatureName.HAND_TRACKING,
          'latest',
          {
            xrInput: defaultXRExperience.input
          }
        )
      }

      let entered = false
      const cameralLocations = []
      const buttonClickingHistory = { key: null, index: 0 }

      defaultXRExperience.baseExperience.sessionManager.onXRFrameObservable.add(
        () => {
          if (!entered) {
            entered = true

            defaultXRExperience.input.xrCamera.position = new Vector3(
              0,
              0.4,
              -0.55
            )
            // defaultXRExperience.input.xrCamera.rotationQuaternion =
            //   Quaternion.RotationAxis(Vector3.Up(), Math.PI / -2)
          }

          cameralLocations.push({
            timeStamp: Date.now(),
            buttonClickingHistory: { ...buttonClickingHistory },
            position: defaultXRExperience.input.xrCamera.position.asArray(),
            rotation:
              defaultXRExperience.input.xrCamera.rotationQuaternion.asArray()
          })
        }
      )

      calibrate(
        defaultXRExperience.input.xrCamera,
        scene,
        calibrationTimeStamps => {
          createEnviroment(scene)
          createQuestionnaire(
            scene,
            buttonClickingHistory,
            cameralLocations,
            calibrationTimeStamps
            // []
          )
        }
      )
    }
    //Render the scene we created
    babylonEngine.runRenderLoop(() => {
      scene.render()
    })
  })

//Listen for window size changes and resize the scene accordingly
window.addEventListener('resize', function () {
  babylonEngine.resize()
})

// hide/show the Inspector
window.addEventListener('keydown', ev => {
  // Shift+Ctrl+Alt+I
  if (ev.shiftKey && ev.ctrlKey && ev.altKey && ev.keyCode === 73) {
    if (scene.debugLayer.isVisible()) {
      scene.debugLayer.hide()
    } else {
      scene.debugLayer.show()
    }
  }
})
