import numpy as np
import pandas as pd

def calc_saccade_len_vel(df: pd.DataFrame) -> list:
    if not df['fixation x [px]'].empty:
        x = df['fixation x [px]'].to_numpy()
        y = df['fixation y [px]'].to_numpy()
        # saccade_length = np.sqrt(np.diff(x)**2 + np.diff(y)**2)
        # saccade_velocity = np.sqrt(np.diff(x)**2 + np.diff(y)**2) / np.diff(df['start timestamp [ns]']) * 1e3 # convert to px/second
        saccade_length = px2angle(x, y)
        saccade_velocity = saccade_length / np.diff(df['start timestamp [ns]']) * 1e3 # convert to degree/second
        return [np.round(np.mean(saccade_length),3), np.round(np.std(saccade_length),3), np.round(np.mean(saccade_velocity),3), np.round(np.std(saccade_velocity),3)]
    return [0, 0, 0, 0]

def px2angle(x, y) -> np.ndarray:
    """
    Convert pixel coordinate (x, y) to angle in degrees.
    
    Args:
        x, y: Pixel coordinates, 1-D array

    Returns:
        np.array: Angle in degrees.
    """
    # see more at https://pupil-labs.com/products/neon/specs
    X_screen = 1600
    Y_screen = 1200
    X_deg = 103 # not sure if this should be 132
    Y_deg = 77 # not sure if this should be 81

    dx_deg = np.diff(x) * (X_deg / X_screen)
    dy_deg = np.diff(y) * (Y_deg / Y_screen)

    return np.sqrt(dx_deg**2 + dy_deg**2)
