import {
  Scene,
  HemisphericLight,
  PointLight,
  ArcRotateCamera,
  Vector3,
  Color3,
  Color4
} from '@babylonjs/core'


//Create and export a function that takes a Babylon engine and returns a Babylon Scene
export async function createScene (engine) {
  //Create an empty Scene
  const scene = new Scene(engine)
  scene.clearColor = new Color4(0.078, 0.231, 0.427, 0.9)

  const pointLight = new PointLight('light', new Vector3(0, 3.4, 0), scene)
  pointLight.intensity = 0.8
  //Add some lighting
  let light = new HemisphericLight('light1', new Vector3(0, 10, 0), scene)
  light.diffuse = new Color3(1, 1, 1)
  light.specular = new Color3(1, 1, 1)
  light.groundColor = new Color3(1, 1, 1)

  //Add a camera that rotates around the origin and adjust its properties
  const camera = new ArcRotateCamera(
    'Camera',
    -(Math.PI / 2),
    0,
    2,
    new Vector3(0, 0.2, -0.75),
    scene
  )
  camera.wheelPrecision = 30 // Adjust the sensitivity of the mouse wheel's zooming
  camera.minZ = 0 // Adjust the distance of the camera's near plane
  camera.attachControl(true) // Allow the camera to respond to user controls
  camera.position = new Vector3(0, 0.2, 0)

  return scene
}
