import * as d3force from 'd3-force-3d'

export default function convertGraph (fileContent) {
  let lines = fileContent.split('\n')
  let edges = lines.filter(s => !s.startsWith('%') && s.length !== 0)
  let nodes = new Set()
  let links = []

  if (lines[0] === '% sym posweighted') {
    for (let edge of edges) {
      let [source, target, weight] = edge.split(' ')

      nodes.add(source - 1)
      nodes.add(target - 1)

      links.push({
        source: source - 1,
        target: target - 1,
        value: +weight
      })
    }
  } else {
    for (let edge of edges) {
      let [source, target] = edge.replace(/\s+/g, ' ').split(' ')
      nodes.add(source - 1)
      nodes.add(target - 1)

      links.push({
        source: source - 1,
        target: target - 1
      })
    }
  }

  nodes = Array.from(nodes.values()).map(i => {
    return { index: i }
  })

  let simulation = d3force
    .forceSimulation(nodes, 3)
    .force('link', d3force.forceLink(links))
    .force('charge', d3force.forceManyBody())
    .force('collide', d3force.forceCollide())
    .force('center', d3force.forceCenter(0, 0, 0))
    .tick(8)

  links = removeDuplicateLinks(links)

  return { nodes, links }
}

function removeDuplicateLinks (links) {
  // Create a new array to store unique links
  const uniqueLinks = []

  // Create a Set to track processed links
  const processedLinks = new Set()

  for (const link of links) {
    // Create a unique identifier for each link (regardless of direction)
    // We'll use the smaller index first to make a consistent identifier
    const sourceIndex = link.source.index
    const targetIndex = link.target.index

    const minIndex = Math.min(sourceIndex, targetIndex)
    const maxIndex = Math.max(sourceIndex, targetIndex)
    const linkId = `${minIndex}-${maxIndex}`

    // If we haven't processed this link before, add it to our result
    if (!processedLinks.has(linkId)) {
      uniqueLinks.push(link)
      processedLinks.add(linkId)
    }
  }

  return uniqueLinks
}
