import '@babylonjs/core/Debug/debugLayer'
import '@babylonjs/inspector'
import { Engine } from '@babylonjs/core'
import { createScene } from './scene.js'
import parseGraph from './importer.js'
import convertGraph from './converter.js'
import generateGraphQuestions from './question'
import { scaleNodesToUnitCube } from '../utils'

//Grab DOM element where we will attach our canvas. #app is the id assigned to an empty <div> in our index.html
const app = document.querySelector('#app')
//Create a canvas element and append it to #app div
const form = document.createElement('form')
form.id = 'uploadForm'

// 创建文件上传 input
const fileInput = document.createElement('input')
fileInput.type = 'file'
fileInput.id = 'fileInput'
// fileInput.accept = '.graphml'

const readButton = document.createElement('button')
readButton.type = 'button'
readButton.textContent = 'Upload Graphml'
readButton.addEventListener('click', readFile)

const canvas = document.createElement('canvas')
form.appendChild(fileInput)
form.appendChild(readButton)

app.appendChild(form)
app.appendChild(canvas)

//initialize babylon engine, passing in our target canvas element, and create a new scene
const babylonEngine = new Engine(canvas, true)

//create a scene object using our engine
let scene
//Render the scene we created
babylonEngine.runRenderLoop(() => {
  scene && scene.render()
})

//Listen for window size changes and resize the scene accordingly
window.addEventListener('resize', function () {
  babylonEngine.resize()
})

// hide/show the Inspector
window.addEventListener('keydown', ev => {
  // Shift+Ctrl+Alt+I
  if (ev.shiftKey && ev.ctrlKey && ev.altKey && ev.keyCode === 73) {
    if (scene.debugLayer.isVisible()) {
      scene.debugLayer.hide()
    } else {
      scene.debugLayer.show()
    }
  }
})

function readFile () {
  const fileInput = document.getElementById('fileInput')

  if (fileInput.files.length === 0) {
    alert('Please upload a fileQ')
    return
  }

  const file = fileInput.files[0]
  const reader = new FileReader()

  reader.onload = function (event) {
    const fileContent = event.target.result.replace(/,/g, '.')
    let graph

    if (file.name.endsWith('.graphml')) {
      graph = parseGraph(fileContent)
    } else {
      graph = convertGraph(fileContent)
    }

    graph.nodes = scaleNodesToUnitCube(graph.nodes)

    graph.links = graph.links.map(link => {
      return {
        index: link.index,
        source: graph.nodes[link.source.index],
        target: graph.nodes[link.target.index]
      }
    })

    const questions = generateGraphQuestions(graph)
    const questionsJson = JSON.stringify(questions)

    const downloadButton = document.createElement('button')
    downloadButton.type = 'button'
    downloadButton.textContent = file.name

    downloadButton.addEventListener('click', event => {
      event.preventDefault()

      const blob = new Blob([questionsJson], { type: 'application/json' })
      const url = URL.createObjectURL(blob)

      const a = document.createElement('a')
      a.href = url
      a.download = file.name.slice(0, file.name.indexOf('.')) + '_question.json'
      a.style.display = 'none'

      document.body.appendChild(a)
      a.click()

      setTimeout(() => {
        document.body.removeChild(a)
        URL.revokeObjectURL(url)
      }, 100)
    })

    form.append(downloadButton)

    createScene(babylonEngine, {
      graph,
      highlightedNodes: []
    }).then(s => (scene = s))
  }

  reader.onerror = function () {
    alert('文件读取失败')
  }

  reader.readAsText(file)
}
