import {
  Vector3,
  MultiMaterial,
  StandardMaterial,
  Color3,
  MeshBuilder,
  SubMesh
} from '@babylonjs/core'

function createEnviroment (scene) {
  const box = MeshBuilder.CreateBox('room', { size: 4 }, scene)
  box.position = new Vector3(0, 1.45, 0.49)

  const topBottomMaterial = new StandardMaterial('topBottomMaterial', scene)
  const lateralMaterial = new StandardMaterial('lateralMaterial', scene)

  topBottomMaterial.diffuseColor = Color3.FromHexString('#0d539d')
  lateralMaterial.diffuseColor = Color3.FromHexString('#002f59')
  topBottomMaterial.backFaceCulling = false
  lateralMaterial.backFaceCulling = false


  const multiMat = new MultiMaterial('multi', scene)
  multiMat.subMaterials.push(topBottomMaterial) 
  multiMat.subMaterials.push(lateralMaterial) 

  const verticesCount = box.getTotalVertices()
  box.subMeshes = []

  // lateral
  box.subMeshes.push(new SubMesh(1, 0, verticesCount, 0, 6, box))
  // lateral
  box.subMeshes.push(new SubMesh(1, 0, verticesCount, 6, 6, box))
  // lateral
  box.subMeshes.push(new SubMesh(1, 0, verticesCount, 12, 6, box))
  // lateral
  box.subMeshes.push(new SubMesh(1, 0, verticesCount, 18, 6, box))
  // topBottom
  box.subMeshes.push(new SubMesh(0, 0, verticesCount, 24, 6, box))
  // topBottom
  box.subMeshes.push(new SubMesh(0, 0, verticesCount, 30, 6, box))

  box.material = multiMat
  return box
}

function scaleNodesToUnitCube (nodes) {
  // If array is empty, return immediately
  if (!nodes || nodes.length === 0) {
    return nodes
  }

  // Find the maximum absolute value in each dimension
  let maxAbsX = 0
  let maxAbsY = 0
  let maxAbsZ = 0

  for (const node of nodes) {
    maxAbsX = Math.max(maxAbsX, Math.abs(node.x))
    maxAbsY = Math.max(maxAbsY, Math.abs(node.y))
    maxAbsZ = Math.max(maxAbsZ, Math.abs(node.z))
  }

  // Find the maximum value across all dimensions to use as scaling factor
  const maxValue = Math.max(maxAbsX, maxAbsY, maxAbsZ)

  // If max value is 0, all points are at origin, no scaling needed
  if (maxValue === 0) {
    return nodes
  }

  // Calculate scaling factor. Target is to scale maximum absolute value to 0.5
  // Since the cube has length 1 and is centered at (0,0,0), the range in each direction is [-0.5, 0.5]
  const scaleFactor = 0.5 / maxValue

  // Scale all points
  for (const node of nodes) {
    node.x *= scaleFactor
    node.y *= scaleFactor
    node.z *= scaleFactor
  }

  return nodes
}

export { createEnviroment, scaleNodesToUnitCube }
